// --- [ main.c ] -------------------------------------------------------------
//
//       tab = 3 - 13.03.2012
//
//  Testreihen fr den MultiMaster
//  fr 3 Controller mit unterschiedliche Adressen und Einstellungen
//  vor dem Kompilieren muss in "global.h"  DEVICE 1/2/3 gewhlt werden

#define EXTERN
#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <stdlib.h>
#include <string.h>
#include <util/delay.h>

#include "global.h"
#include "TWI_Multi_Master.h"

#define TRUE   1
#define FALSE  0

// --- public - Variablen -----------------------------------------------------

uint8_t TWI_MA_Job = FALSE;
volatile uint8_t wait_time = 1;

extern uint8_t TWI_Arbitration_State;

// --- Prototypen -------------------------------------------------------------

// ----------------------------------------------------------------------------
ISR(TIMER2_COMPA_vect)
{
   wait_time = 0;
}

// ----------------------------------------------------------------------------
int main(void)
{
   uint8_t rx, r;
   uint8_t twi_buffer[20];                   // In TWI_Master.h die Gre des Arrays (TWI_BUF_SIZE) prfen
   uint8_t buffer[20];

   uint8_t job = 5, i = 0, j;

   TWI_MM_Init();                            // Die TWI-Schnittstelle initialisieren
   asm volatile ("sei");                     // Interrupts freigeben

   DDRB = (1<<PB0 | 1<<PB1);
   PORTB = (1<<PB0 | 1<<PB1);

   TCCR2A = (1<< WGM21);                     // CTC-Modus
   TCCR2B = (1<<CS22 | 1<< CS21 | 1 << CS20);  // Prescale: 1024 / 4.000.000
   OCR2A = 26 + TWI_SLA_ADR/4;                 // unterchiedliche Comparewerte fr die Master
// im Mittel 26 + 4 = 30
   // 36 + (2 + 4 + 8)/ 3 = 40 ->  (40 * 1024) / 4.000.000 =  10ms -> duty 30%
   // 26 + (2 + 4 + 8)/ 3 = 30 ->  (30 * 1024) / 4.000.000 =  7.7ms -> duty ca. 40%
   //TIMSK2 = (1<<OCIE2A);

   while(1)
   {
      if((rx = TWI_SLA_Rx_Cnt()))            // Wenn der Slave Daten empfangen hat
      {

         PINB = (1<<PB0);
         TWI_SLA_Get_Data(twi_buffer);       // die Daten abholen

         if ((rx == 1))
         {
            TWI_MA_Job = TRUE;                    // die eigene Adresse oder 0 startet den Job
         }
                                                // eigene Adresse / 0 + Jobnummer + "#"
         else if ((rx == 3) && (twi_buffer[1] < 11) && (twi_buffer[2] == '#'))
         {
            job = twi_buffer[1];
            PINB= (1<<PB1);
         }                                      // eigene Adresse / 0 + eigene Adresse startet Stresstest
         else if ((rx == 2) && (twi_buffer[1] == (TWI_SLA_ADR & 0xFE)))
         {
            TIMSK2 = (1<<OCIE2A);

         }                                     // eigene Adresse / 0 + eigene Adresse+1 stoppt Stresstest
         else if ((rx == 2) && (twi_buffer[1] == (TWI_SLA_ADR | 0x01)))
         {
            TIMSK2 = 0;
         }

 //      for(i = 0; i < rx; i++) twi_buffer[i] *= 2;

         TWI_SLA_Put_Data(twi_buffer, rx);  // die empfangenen Daten in den Sendepuffer stellen
#if DEBUG == TRUE
         TWI_Clear_States();
#endif
      }
/*
      if (wait_time == 0)                    // Blinken zeigt an, dass das Programm luft ...
      {
         wait_time++;
         PINB = (1<<PB0);
      }
*/

   // --- hier wird ein Belastungstest ausgefhrt
      if (wait_time == 0)
      {
         wait_time = 1;
         twi_buffer[0] = 64;
         twi_buffer[1] = 97;
         twi_buffer[2] = 98;
         twi_buffer[3] = i;

         if (!(TWI_MM_Write_Read(twi_buffer, 4, 4))) // 8 Byte, Dauer ca. 1ms
         {
           TIMSK2 = 0;
           while(1)
           {
               _delay_ms(200);
               PORTB |= (1<<PB0);
           }
         }
   // prfen, ob die gesendeten Daten auch wieder eingelesen wurden
         if (!(twi_buffer[1] == i))
         {
            TIMSK2 = 0;

           while(1)
           {
               _delay_ms(100);
               PORTB |= (1<<PB0);
           }
         }
         i++;
         continue;
      }


   // -- hier werden einzelne Test ausgefhrt, die Variable job definiert den Test

      if (TWI_MA_Job)                          // wenn der Master eine Aktion ausfhren soll
      {
        TWI_MA_Job = 0;                         // nur einmal ausfhren

// #######################################################
// - alle Master schreiben die selben Daten auf den PCF8574
if (job == 1)
{
   buffer[0] = 64;           // sendet an breadboard_2
   buffer[1] = 0x55;
   buffer[2] = 0xAA;
   r =    TWI_MM_Write(buffer, 3 );
}

// - alle lesen vom PCF8574
else if (job == 2)
{
   buffer[0] = 64;           // sendet an breadboard_2
   r =    TWI_MM_Read(buffer, 3 );
}

// - alle schreiben unterschiedliche Daten auf den PCF8574
if (job == 3)
{
   buffer[0] = 64;           // sendet an breadboard_2

   #if DEVICE == breadboard
      buffer[1] = 41;
      buffer[2] = 42;
   #elif DEVICE == breadboard_2
      buffer[1] = 81;
      buffer[2] = 82;
   #elif DEVICE == platine
      buffer[1] = 21;
      buffer[2] = 22;
   #endif

   r = 4;
   while(r--)
   {
   if (TWI_MM_Write(buffer, 3 )) break;
   }
}

// die drei Master senden unterschiedliche Daten an den PCF8574,
// senden ein REPEATED START und lesen die Daten wieder aus

else if (job == 4)
{
      buffer[0] = 64;
   #if DEVICE == breadboard
      buffer[1] = 43;
      buffer[2] = 44;
   #elif DEVICE == breadboard_2
      buffer[1] = 83;
      buffer[2] = 84;
   #elif DEVICE == platine
      buffer[1] = 23;
      buffer[2] = 24;
   #endif
   /*
j = 5;
while(j--)

   TWI_MM_Set_Stop_Mode(TWI_REPEATED_START);
   if (!(TWI_MM_Write(buffer, 3))) continue;
   TWI_MM_Set_Stop_Mode(TWI_STOP);

   if (!(TWI_MM_Read(buffer, 3))) continue;
   else break;
}
*/
   TWI_MM_Set_Stop_Mode(TWI_REPEATED_START);
   TWI_MM_Write(buffer, 3);

   TWI_MM_Set_Stop_Mode(TWI_STOP);
   TWI_MM_Read(buffer, 3);

}
// ########################################################################

// Die Master senden 2 Bytes an unterschiedliche Adressen eines Eeprom

else if (job == 5)
{

   #if DEVICE == breadboard
      buffer[0] = 162;
      buffer[1] = 2;
      buffer[2] = 4;
      buffer[3] = 44;
      buffer[4] = 45;
   #elif DEVICE == breadboard_2
      buffer[0] = 162;
      buffer[1] = 2;
      buffer[2] = 6;
      buffer[3] = 66;
      buffer[4] = 67;
   #elif DEVICE == platine
      buffer[0] = 162;
      buffer[1] = 2;
      buffer[2] = 2;
      buffer[3] = 22;
      buffer[4] = 23;
   #endif

/*
j = 7;
while(j--)
   {
   if (TWI_MM_Write(buffer, 5)) break; // Adresse + 2 Bytes
   }

*/
 TWI_MM_Write(buffer, 5);     // Adresse + 2 Bytes
}

// senden ein Write_Read an das Eeprom um Daten auszulesen
else if (job == 6)
{

   #if DEVICE == breadboard
      buffer[0] = 162;
      buffer[1] = 2;
      buffer[2] = 4;
      buffer[3] = 44;
      buffer[4] = 45;
   #elif DEVICE == breadboard_2
      buffer[0] = 162;
      buffer[1] = 2;
      buffer[2] = 6;
      buffer[3] = 88;
      buffer[4] = 89;
   #elif DEVICE == platine
      buffer[0] = 162;
      buffer[1] = 2;
      buffer[2] = 2;
      buffer[3] = 22;
      buffer[4] = 23;
   #endif

/*
j = 7;
while(j--)
   {
   if (TWI_MM_Write_Read(buffer, 3, 3)) break;
   }
*/
   TWI_MM_Write_Read(buffer, 3, 3);

}

// #####################################################################


// die drei Master senden sich gegenseitig Daten zu
else if (job == 7)
{
   #if DEVICE == breadboard
      buffer[0] = 32;           // sendet an breadboard_2
      buffer[1] = 81;
      buffer[2] = 82;
   #elif DEVICE == breadboard_2
      buffer[0] = 8;           // sendet an Platine
       buffer[1] = 21;
      buffer[2] = 22;
   #elif DEVICE == platine
      buffer[0] = 16;           // sendet an breadboard
       buffer[1] = 41;
      buffer[2] = 42;
   #endif
/*
  r = 4;
   while(r--)
   {
   if( TWI_MM_Write(buffer, 3 )) break;
   }
*/
 TWI_MM_Write(buffer, 3 );

}

// die drei Master holen gegenseitig Daten voneinander ab
// benutzt die Funktion
else if (job == 8)
{
   #if DEVICE == breadboard
      buffer[0] = 32;           // sendet an breadboard_2
   #elif DEVICE == breadboard_2
      buffer[0] = 8;           // sendet an Platine
   #elif DEVICE == platine
      buffer[0] = 16;           // sendet an breadboard
   #endif

/*
r = 4;
while(r--)
{
   if (TWI_MM_Read(buffer, 3 )) break;
   }

*/

TWI_MM_Read(buffer, 3 );
}

// die drei Master senden 2 Byte an den Nachfolger
// und lesen gleich danach 2 Byte wieder aus
// benutzt die Funktion ReadWrite()
else if (job == 9)
{
   #if DEVICE == breadboard
      buffer[0] = 32;           // sendet an breadboard_2
      buffer[1] = 81;
      buffer[2] = 82;
   #elif DEVICE == breadboard_2
      buffer[0] = 8;           // sendet an Platine
      buffer[1] = 21;
      buffer[2] = 22;
   #elif DEVICE == platine
      buffer[0] = 16;           // sendet an breadboard
      buffer[1] = 41;
      buffer[2] = 42;
   #endif

/*
j = 4;
while(j--)
{
   if (!(TWI_MM_Write_Read(buffer, 3, 3 ))) continue;
   else break;
}
*/
   TWI_MM_Write_Read(buffer, 3, 3 );

}


// die selbe Funktion wie Vorgnger, aber mit manuellem Repeated Start
// die drei Master schreiben unterschiedliche Daten jeweils zum benachbarten
// Master, senden ein Repeated Start und lesen die Daten danach wieder aus
else if (job == 10)
{
   #if DEVICE == breadboard
      buffer[0] = 32;           // sendet an breadboard_2
       buffer[1] = 85;
      buffer[2] = 86;
      buffer[3] = 87;
   #elif DEVICE == breadboard_2
      buffer[0] = 8;           // sendet an Platine
      buffer[1] = 25;
      buffer[2] = 26;
      buffer[3] = 27;
   #elif DEVICE == platine
      buffer[0] = 16;           // sendet an breadboard
      buffer[1] = 45;
      buffer[2] = 46;
      buffer[3] = 47;
    #endif
/*
j = 4;
while(j--)
{
   TWI_MM_Set_Stop_Mode(TWI_REPEATED_START);
   if (!(r = TWI_MM_Write(buffer, 4))) continue ;
   TWI_MM_Set_Stop_Mode(TWI_STOP);
   if (!(r = TWI_MM_Read(buffer, 4))) continue;
   if (r) break;
}
*/

   TWI_MM_Set_Stop_Mode(TWI_REPEATED_START);
   TWI_MM_Write(buffer, 4);

   TWI_MM_Set_Stop_Mode(TWI_STOP);
   TWI_MM_Read(buffer, 4);

}



#if DEBUG == TRUE
// gibt TWSR-Code auf dem Logger aus
   _delay_ms(TWI_SLA_ADR * 4);
   TWI_Get_States();
#endif

      }

   }

}
// --- [ eof ] ----------------------------------------------------------------
