// --- [ TWI_Multi_Master.h ] -------------------------------------------------

/*****************************************************************************
*
* File              : TWI_Multi_Master.h
*
* AppNote           : AVR311 / 315
*
* Description       : Header file for TWI_slave.c
*                     Include this file in the application.
*
****************************************************************************/

#include "global.h"

#if DEVICE == breadboard
   #define TWI_SLA_ADR  (16 | 1)       // +1 fr General Call

#elif DEVICE == breadboard_2
   #define TWI_SLA_ADR  (32 | 1)       // +1 fr General Call

#elif DEVICE == platine
   #define TWI_SLA_ADR  (8 | 1)        // +1 fr General Call

#else
   #error ERROR device
#endif

#define MA_BUF_SIZE 	   16             // Puffergre fr den Master (incl. Adressbyte)
#define SLA_RX_BUF_SIZE	16					// Puffergre fr den Slave (incl. Adressbyte)
#define SLA_TX_BUF_SIZE	16				   // Puffergre fr den Slave (kein Adressbyte erforderlich)

#define ERROR_RETRYS      5           // Nach einem Fehler werden 'n' neue Versuche gestartet
#define ERROR_WAIT_CYCLES 100          // max = 127 (ERROR_WAIT_CYCLES + TWI_ADR/ 2) WAIT_US im Microsekunden
#define WAIT_US            3           // die Konstante fr _delay_us in der Warteschleife

#define TWI_CLOCK      	100000ul			// 100kHz
#define TWI_TWBR 	((F_CPU / (2 * TWI_CLOCK)) - 8)

#define TWI_BUSY           (TWCR & (1<<TWIE))
#define TWI_Get_State()    (TWSR)

#define TWI_REPEATED_START 1
#define TWI_STOP           0


// --- Prototypen ------------------------------------------------------------

// --- Allgemein -------------------------------------------------------------
void 	  TWI_MM_Init( void );
void    TWI_Get_States(void);          // nur zum Debuggen
void    TWI_Clear_States(void);        // nur zum Debuggen
//uint8_t  TWI_Get_State(void);        // als Makro definiert

// --- Slave-Modus ------------------------------------------------------------
uint8_t TWI_SLA_Rx_Cnt(void);
void    TWI_SLA_Put_Data( uint8_t *msg, uint8_t msgSize );
void    TWI_SLA_Get_Data( uint8_t *msg);

// --- Master-Modus -----------------------------------------------------------
uint8_t TWI_MM_Read( uint8_t *msg, uint8_t msgSize );
uint8_t TWI_MM_Write( uint8_t *msg, uint8_t msgSize );
uint8_t TWI_MM_Write_Read(uint8_t *, uint8_t out, uint8_t in);
void    TWI_MM_Set_Stop_Mode(uint8_t);

// --- [ eof ] ----------------------------------------------------------------
